//
//  OJSBlockSetupExtension.swift
//  OJASwiftKit
//
//  Created by luxiaoming on 2018/7/3.
//  Copyright © 2018年 luxiaoming. All rights reserved.
//

import UIKit
import AVFoundation

extension UIInterfaceOrientation {
    
    static func blockSetup<T>(for interfaceOrientation: UIInterfaceOrientation = UIApplication.shared.statusBarOrientation, protraitBlock: (() -> T), portraitUpsideDownBlock: () -> T, landscapeLeftBlock: () -> T, landscapeRightBlock: () -> T, unknownBlock: () -> T) -> T {
        switch interfaceOrientation {
        case .portrait: return protraitBlock()
        case .portraitUpsideDown: return portraitUpsideDownBlock()
        case .landscapeLeft: return landscapeLeftBlock()
        case .landscapeRight: return landscapeRightBlock()
        case .unknown: return unknownBlock()
        }
    }
}

extension AVCaptureVideoOrientation {
    static func blockSetup<T>(for videoOrientation: AVCaptureVideoOrientation, protraitBlock: (() -> T), portraitUpsideDownBlock: () -> T, landscapeLeftBlock: () -> T, landscapeRightBlock: () -> T) -> T {
        switch videoOrientation {
        case .portrait: return protraitBlock()
        case .portraitUpsideDown: return portraitUpsideDownBlock()
        case .landscapeLeft: return landscapeLeftBlock()
        case .landscapeRight: return landscapeRightBlock()
        }
    }
}

extension AVCaptureDevice.Position {
    static func blockSetup<T>(for cameraPosition: AVCaptureDevice.Position, frontBlock: () -> T, backBlock: () -> T,  unspecifiedBlock: () -> T) -> T {
        switch cameraPosition {
        case .front: return frontBlock()
        case .back: return backBlock()
        case .unspecified: return unspecifiedBlock()
        }
    }
}
