//
//  OJSNibExtension.swift
//  OJASwiftKitDemo
//
//  Created by luxiaoming on 2016/11/14.
//  Copyright © 2016年 luxiaoming. All rights reserved.
//

import UIKit
import Foundation

private var ojs_absoluteClassNameKey = "ojs_absoluteClassName"

extension NSObject {
    public class func ojs_absoluteClassName() -> String {
        if let name = objc_getAssociatedObject(self, &ojs_absoluteClassNameKey) as? String {
            return name
        }
        let classString = NSStringFromClass(self.classForCoder())
        let array = classString.components(separatedBy: ".")
        let name = array.last!
        objc_setAssociatedObject(self, &ojs_absoluteClassNameKey, name, .OBJC_ASSOCIATION_COPY_NONATOMIC)
        return name
    }
}

extension UICollectionReusableView {
    public static var ojs_reuseIdentifier: String {
        return self.ojs_absoluteClassName()
    }
}

extension UITableViewCell {
    public static var ojs_reuseIdentifier: String {
        return self.ojs_absoluteClassName()
    }
}

extension UITableViewHeaderFooterView {
    public static var ojs_reuseIdentifier: String {
        return self.ojs_absoluteClassName()
    }
}

extension UIView {
    
    public class func ojs_nib(bundle: Bundle? = nil) -> UINib {
        return UINib(nibName: self.ojs_absoluteClassName(), bundle: bundle)
    }
    
    public class func ojs_loadFromNib(bundle: Bundle? = nil) -> Self {
        return loadFromNibHelper(bundle: bundle)
    }
    
    
    private class func loadFromNibHelper<T>(bundle: Bundle? = nil) -> T {
        let bundle = bundle ?? Bundle.main
        let array = bundle.loadNibNamed(self.ojs_absoluteClassName(), owner: nil, options: nil)
        return array?.last as! T
    }
    
}


extension UIViewController {
    
    /// 如果identifier与类名相同，则可以不传
    public class func ojs_loadFromStoryboard(storyboardName: String, identifier: String? = nil, bundle: Bundle? = nil) -> Self {
        return loadFromStroyboardHelper(storyboardName: storyboardName, identifier: identifier, bundle: bundle)
    }
    
    
    private class func loadFromStroyboardHelper<T>(storyboardName: String, identifier: String?, bundle: Bundle? = nil) -> T {
        let storyboard = UIStoryboard(name: storyboardName, bundle: bundle)
        let temp = identifier ?? self.ojs_absoluteClassName()
        return storyboard.instantiateViewController(withIdentifier: temp) as! T
    }
}



extension UITableView {
    
    public func ojs_registerCell(class className: AnyClass) {
        if let className = className as? UITableViewCell.Type {
            self.register(className, forCellReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_registerCell(nib className: AnyClass) {
        if let className = className as? UITableViewCell.Type {
            self.register(className.ojs_nib(), forCellReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_dequeueReusableCell<T>(withClass classType: T.Type) -> T where T: UITableViewCell {
        let cell = self.dequeueReusableCell(withIdentifier: classType.ojs_reuseIdentifier) as! T
        return cell
    }
    
    
    
    public func ojs_registerHeaderFooterView(class className: AnyClass) {
        if let className = className as? UITableViewHeaderFooterView.Type {
            self.register(className, forHeaderFooterViewReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_registerHeaderFooterView(nib className: AnyClass) {
        if let className = className as? UITableViewHeaderFooterView.Type {
            self.register(className.ojs_nib(), forHeaderFooterViewReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_dequeueReusableHeaderFooterView<T>(withClass classType: T.Type) -> T where T: UITableViewHeaderFooterView {
        let view = self.dequeueReusableHeaderFooterView(withIdentifier: classType.ojs_reuseIdentifier) as! T
        return view
    }
}


extension UICollectionView {
    public func ojs_registerCell(class className: AnyClass) {
        if let className = className as? UICollectionViewCell.Type {
            self.register(className, forCellWithReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_registerCell(nib className: AnyClass) {
        if let className = className as? UICollectionViewCell.Type {
            self.register(className.ojs_nib(), forCellWithReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_dequeueReusableCell<T>(withClass classType: T.Type, for indexPath: IndexPath) -> T where T: UICollectionViewCell {
        let cell = self.dequeueReusableCell(withReuseIdentifier: classType.ojs_reuseIdentifier, for: indexPath) as! T
        return cell
    }
    
    
    
    
    public func ojs_registerSupplementaryView(class className: AnyClass, kind: String) {
        if let className = className as? UICollectionReusableView.Type {
            self.register(className, forSupplementaryViewOfKind: kind, withReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_registerSupplementaryView(nib className: AnyClass, kind: String) {
        if let className = className as? UICollectionReusableView.Type {
            self.register(className.ojs_nib(), forSupplementaryViewOfKind: kind, withReuseIdentifier: className.ojs_reuseIdentifier)
        }
    }
    
    public func ojs_dequeueReusableSupplementaryView<T>(ofKind kind: String, withClass classType: T.Type, for indexPath: IndexPath) -> T where T: UICollectionReusableView {
        let supplementaryView = self.dequeueReusableSupplementaryView(ofKind: kind, withReuseIdentifier: classType.ojs_reuseIdentifier, for: indexPath) as! T
        return supplementaryView
    }
    
}
