//
//  OJSScrollViewExtension.swift
//  OJASwiftKitDemo
//
//  Created by luxiaoming on 2017/8/12.
//  Copyright © 2017年 luxiaoming. All rights reserved.
//

import Foundation
import UIKit


extension UIScrollView {
    
    /// 滑动到顶部，注意：没有delay == 0时会立即执行，但contentSize可能会不准
    open func scrollToTop(afterDelay delay: Double = 0.1, animated: Bool = true) {
        let topRect = CGRect(x: 0, y: 0, width: self.frame.width, height: 1)
        if delay > Double.leastNormalMagnitude {
            Delay(delay) {
                self.scrollRectToVisible(topRect, animated: animated)
            }
        } else {
            self.scrollRectToVisible(topRect, animated: animated)
        }
        
        
    }
    
    /// 滑动到底部，注意：没有delay == 0时会立即执行，但contentSize可能会不准
    /// 注意：iOS11以后，tableView的contentSize不是一开始就知道的，而是随着拖动慢慢计算出来的，所以scrollRectToVisible方法就失效了
    /// 只能用其他的方式
    open func scrollToBottom(afterDelay delay: Double = 0.1, animated: Bool = true) {
        if let tableView = self as? UITableView {
            let lastSection = tableView.numberOfSections - 1
            guard lastSection >= 0 else { return }
            let lastRow = tableView.numberOfRows(inSection: lastSection) - 1
            guard lastRow >= 0 else { return }
            let indexPath = IndexPath(row: lastRow, section: lastSection)
            if delay > Double.leastNormalMagnitude {
                Delay(delay) {
                    tableView.scrollToRow(at: indexPath, at: .bottom, animated: animated)
                }
            } else {
                tableView.scrollToRow(at: indexPath, at: .bottom, animated: animated)
            }
        }
        else if let collectionView = self as? UICollectionView {
            let lastSection = collectionView.numberOfSections - 1
            guard lastSection >= 0 else { return }
            let lastItem = collectionView.numberOfItems(inSection: lastSection) - 1
            guard lastItem >= 0 else { return }
            let indexPath = IndexPath(item: lastItem, section: lastSection)
            if delay > Double.leastNormalMagnitude {
                Delay(delay) {
                    collectionView.scrollToItem(at: indexPath, at: .bottom, animated: animated)
                }
            } else {
                collectionView.scrollToItem(at: indexPath, at: .bottom, animated: animated)
            }
        }
        else {
            let bottomRect = CGRect(x: 0, y: self.contentSize.height - 1, width: self.frame.width, height: 1) //注意，如果目标rect的位置超出contentSize，就什么都不会执行
            if delay > Double.leastNormalMagnitude {
                Delay(delay) {
                    self.scrollRectToVisible(bottomRect, animated: animated)
                }
            } else {
                self.scrollRectToVisible(bottomRect, animated: animated)
            }
        }
    }
    
}



extension UITableView {
    
    open func ojs_scrollToRow(at indexPath: IndexPath, at scrollPosition: UITableView.ScrollPosition, animated: Bool) {
        let sectionCount = self.numberOfSections
        guard sectionCount > 0 && indexPath.section >= 0 && indexPath.section <= sectionCount - 1 else { return }
        let rowCount = self.numberOfRows(inSection: indexPath.section)
        guard rowCount > 0 && indexPath.row >= 0 && indexPath.row <= rowCount - 1 else { return }
        self.scrollToRow(at: indexPath, at: scrollPosition, animated: animated)
    }
    
}

extension UICollectionView {
    
    open func ojs_scrollToItem(at indexPath: IndexPath, at scrollPosition: UICollectionView.ScrollPosition, animated: Bool) {
        let sectionCount = self.numberOfSections
        guard sectionCount > 0 && indexPath.section >= 0 && indexPath.section <= sectionCount - 1 else { return }
        let itemCount = self.numberOfItems(inSection: indexPath.section)
        guard itemCount > 0 && indexPath.row >= 0 && indexPath.row <= itemCount - 1 else { return }
        self.scrollToItem(at: indexPath, at: scrollPosition, animated: animated)
    }
}
